/*
	Copyright (C) 2008-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include <unistd.h>
#include <time.h>
#include <errno.h>

#include <mtkit.h>
#include <mtpixel.h>
#include <mtcelledit.h>
#include <mtcedui.h>



#ifdef U_NLS
	#include <libintl.h>
	#define _(text) gettext(text)
	#define __(text) gettext(text)
#else
	#define _(text) text
	#define __(text) text
#endif



#define PREFS_WINDOW_X		CED_INIFILE_MAIN_WINDOW"_x"
#define PREFS_WINDOW_Y		CED_INIFILE_MAIN_WINDOW"_y"
#define PREFS_WINDOW_W		CED_INIFILE_MAIN_WINDOW"_w"
#define PREFS_WINDOW_H		CED_INIFILE_MAIN_WINDOW"_h"
#define RECENT_MENU_TOTAL	20
#define	CEDVIEW_FRZ_PANE_TOT	7
#define MTCELLEDIT_CLIP_CODE	123456
#define MTCELLEDIT_CLIP_NAME	"application/x-mtcelledit-clipboard"
#define MAX_SORT		10
#define COLOR_SWATCH_SIZE	24
#define COLOR_SWATCH_ROWS	5
#define COLOR_SWATCH_COLS	8
#define COLOR_SWATCH_TOTAL	40
#define FIND_MAX_MATCHES	10000



enum
{
	SORT_TABLE_ROWCOL,
	SORT_TABLE_DIRECTION,
	SORT_TABLE_CASE_SENSITIVE,

	SORT_TABLE_TOTAL
};

enum
{
	SORT_ROWS	= 0,
	SORT_COLUMNS	= 1
};

enum
{
	CEDVIEW_AREA_TL,
	CEDVIEW_AREA_TR,		// Main drawing areas
	CEDVIEW_AREA_BL,
	CEDVIEW_AREA_BR,		// Scrollable area

	CEDVIEW_AREA_CORNER,		// Top left corner area - visual bell

	CEDVIEW_TITLE_C1,
	CEDVIEW_TITLE_C2,		// Row/column title header areas
	CEDVIEW_TITLE_R1,
	CEDVIEW_TITLE_R2,

	CEDVIEW_FRZ_COL,		// Frozen pane areas
	CEDVIEW_FRZ_ROW,
	CEDVIEW_FRZ_V_TOP,
	CEDVIEW_FRZ_V_BOTTOM,
	CEDVIEW_FRZ_H_LEFT,
	CEDVIEW_FRZ_H_RIGHT,
	CEDVIEW_FRZ_CORNER,

	CEDVIEW_AREA_TOTAL
};



void prefs_init (		// Setup memory, filename, etc.
	void		* ptr,
	const char	* filename
	);
void prefs_load ();		// Load prefs file from disk into memory
void prefs_save ();		// Save current prefs from memory to disk
void prefs_close ();		// Free all memory created in prefs_init ()

mtPrefs * prefs_file ();

void prefs_set_int (
	const char	* key,
	int		value
	);

void prefs_set_string (
	const char	* key,
	const char	* value
	);

void prefs_set_double (
	const char	* key,
	double		value
	);

int prefs_get_int (
	const char	* key
	);

char * prefs_get_string (
	const char	* key
	);

double prefs_get_double (
	const char	* key
	);

const char * be_cline (
	int		argc,
	char		** argv
	);

const char * be_cline_filename ();	// File to load from command line

int be_snip_filename (			// Snip input string to fit recently
					// used list.  Length set by
					// CED_INIFILE_RECENT_FILENAME_LEN.
	int		num,		// 1..20
	char		* buf,		// Output buffer
	int		buflen		// Bytes available in buffer
	);
	// 0 = Buffer filled
	// 1 = Buffer not filled

int be_titlebar_text (
	CedUI_file	* file,
	char		* buf,		// Put title bar text here
	int		buflen,
	int		changed
	);
	// 0 = file isn't a book
	// 1 = file is a book



// NOTE: The functions below are implemented by the toolkit code

void pref_change_font (
	mtPrefItemValue	* piv,
	int		callback_data,
	void		* callback_ptr
	);

void pref_change_row_pad (
	mtPrefItemValue	* piv,
	int		callback_data,
	void		* callback_ptr
	);

void pref_change_recent_filename_len (
	mtPrefItemValue	* piv,
	int		callback_data,
	void		* callback_ptr
	);

void pref_change_graph_scale (
	mtPrefItemValue	* piv,
	int		callback_data,
	void		* callback_ptr
	);

// NOTE: end



void be_remember_last_dir (
	char		* filename
	);

int be_register_project (
	CedUI_file	* file,
	char		* filename,
	int		change_project_filename
	);

char * be_get_recent_filename (
	int		i		// 1..20
	);

int be_cedrender_set_font_width (
	CedRender	* render
	);
	// 0 = font_width set

void be_cedrender_set_header_width (
	CedRender	* render,
	int		max
	);

void be_sheet_ref (
	CedSheet	* sheet,
	char		* buf,		// Fill buffer with sheet cell ref.
	int		buflen
	);

void be_quicksum_label (
	CedSheet	* sheet,
	char		* buf,
	int		buflen,
	int		operation
	);

const char * be_get_error_update_text (
	int		error,
	char		* buf,
	int		buflen
	);

void be_update_file_to_book (
	CedUI_file	* file
	);

int be_clip_clear_selection (
	CedUI_file	* file,
	CedSheet	* sheet,
	int		mode
	);

CedSheet * be_clip_transform_start (
	CedUI_clip	* clip,
	int		mode
	);

int be_clip_transform_finish (
	CedUI_clip	* clip,
	CedSheet	* sheet,
	int		mode
	);

int be_clip_copy_values (
	CedSheet	* sheet
	);

int be_clip_copy_output (
	CedSheet	* sheet
	);

int be_selection_row_extent (
	CedSheet	* sheet,
	int		* row,
	int		* rowtot
	);

int be_selection_col_extent (
	CedSheet	* sheet,
	int		* col,
	int		* coltot
	);

int be_fix_years (
	CedUI_file	* file
	);

int be_export_sheet (
	CedSheet	* src,
	char		* filename,
	CedFileType	filetype
	);



// NOTE: The functions below are implemented by the toolkit code

void fe_commit_prefs_set (
	int		pref_id,
	int		pref_num,
	char		* pref_charp,
	int		change_cursor,
	void		* callback_ptr
	);

void fe_save_pref_window_prefs (
	mtPrefs		* prefs
	);

mtPrefs * fe_load_pref_window_prefs (
	mtPrefItem	* table
	);

// NOTE: end

int be_commit_prefs_set (
	CedSheet	* sheet,
	CuBook		* cubook,
	int		pref_id,
	int		pref_num,
	char		* pref_charp
	);

int be_commit_prefs_set_check (
	int		res,
	CedSheet	* sheet,
	int		change_cursor,
	int		pref_id
	);

int be_cellpref_bulk_import (
	mtPrefs		* prefs,
	CedCellPrefs	* cell_prefs
	);

void be_cellpref_changed (
	mtPrefItemValue	* piv,
	int		callback_data,
	void		* callback_ptr
	);

mtPrefs * be_cellpref_init (
	CedSheet	* sheet,
	mtPrefCB	callback,
	void		* ptr
	);

int be_prepare_prefs_set (
	CedSheet	* sheet
	);

void be_cellpref_cleanup (
	CedSheet	* sheet
	);

void be_save_pref_window_prefs (
	mtPrefs		* prefs
	);

void be_load_pref_window_prefs (
	mtPrefs		* prefs
	);

mtPrefs * be_book_prefs_init (
	CedBook		* book
	);

void be_book_prefs_finish (
	mtPrefs		* prefs,
	CedBook		* book
	);

int be_color_swatch_get (
	int		i
	);

char * be_graph_new (
	CedBook		* book
	);

char * be_graph_duplicate (
	CuBook		* cubook
	);

int be_graph_selection_clip (
	CedSheet	* sheet,
	char		* buf,
	int		buflen
	);

void be_find (
	CedUI_file	* file,
	CedSheet	* sheet,
	char		* txt,
	int		wildc,
	int		casen,
	int		mval,
	int		allsheets,
	CedFuncScanArea	callback,
	void		* user_data
	);

