/*
	Copyright (C) 2011-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "be.h"



#define MAX_GRAPH_ID		10000



char * be_graph_new (
	CedBook		* book
	)
{
static char		* graph_text_default =
"/*\n"
"	mtCellEdit Graph Template\n"
"\n"
"	Created		%i-%i-%i %02i:%02i:%02i\n"
"*/\n"
"\n"
"\n"
"{ page		width=\"640\"	height=\"320\"\n"
"			line_color=\"-1\"	fill_color=\"-1\"\n"
"}\n"
"\n"
"{ graph		fill_color=\"0xffffff\"	line_color=\"0\"\n"
"			x_pad=\"10\"	y_pad=\"5\"\n"
"			antialias=\"0\"\n"
"}\n"
"{ plot		fill_color=\"0xdddddd\"\n"
"			x_pad=\"5\"	y_pad=\"5\"	line_color=\"0\"\n"
"			antialias=\"0\"\n"
"}\n"
"\n"
"\n"
"{ x_axis		min=\"0\"		max=\"100\"	}\n"
"{ y_axis		min=\"10\"		max=\"-10\"	}\n"
"\n"
"\n"
"{ plot_x_axis_grid	gap=\"1\"	line_color=\"rgb(200,200,200)\"	antialias=\"0\"	}\n"
"{ plot_y_axis_grid	gap=\"2\"	line_color=\"rgb(200,200,200)\"	antialias=\"0\"	}\n"
"\n"
"{ plot_x_axis_grid	gap=\"5\"	line_color=\"rgb(150,150,150)\"	antialias=\"0\"	}\n"
"{ plot_y_axis_grid	gap=\"10\"	line_color=\"rgb(150,150,150)\"	antialias=\"0\"	}\n"
"\n"
"\n"
"{ plot_x_axis_top	text=\"Graph Title\"	text_size=\"20\"\n"
"				x_justify=\"0.5\"		y_justify=\"0.5\"	y_pad=\"10\"\n"
"}\n"
"\n"
"{ plot_x_axis	size=\"5\"	gap=\"10\"	text_size=\"10\"\n"
"			x_justify=\"0\"		y_pad=\"5\"\n"
"			antialias=\"0\"\n"
"}\n"
"{ plot_y_axis	size=\"5\"	gap=\"2\"	text_size=\"10\"\n"
"			y_justify=\"0\"		x_pad=\"5\"\n"
"			antialias=\"0\"\n"
"}\n"
"\n"
"{ plot_x_axis	text=\"Input\"		text_size=\"14\"\n"
"			y_pad=\"0\"\n"
"}\n"
"{ plot_y_axis	text=\"Output\"		text_size=\"14\"\n"
"			x_pad=\"0\"\n"
"}\n"
"\n"
"\n"
;
	char		graph_name[256],
			* text,
			txt[2048];
	int		i;
	time_t		now;
	struct tm	* now_tm;


	now = time ( NULL );
	now_tm = localtime ( &now );

	for ( i = 1; i < MAX_GRAPH_ID; i++ )
	{
		sprintf ( graph_name, "Graph %i", i );
		if ( ! ced_graph_get ( book, graph_name ) )
		{
			break;
		}

		// Keep looping until we find an unused name
	}

	snprintf ( txt, sizeof ( txt ),
		graph_text_default, now_tm->tm_year + 1900,
		now_tm->tm_mon + 1, now_tm->tm_mday, now_tm->tm_hour,
		now_tm->tm_min, now_tm->tm_sec );

	txt [ sizeof ( txt ) - 1 ] = 0;

	text = strdup ( txt );

	if (	! text			||
		i == MAX_GRAPH_ID	||
		ced_graph_new ( book, text, strlen ( text ), graph_name ) ==
			NULL
		)
	{
		free ( text );

		return NULL;
	}

	mtkit_strfreeswap ( &book->prefs.active_graph, graph_name );

	return book->prefs.active_graph;
}

char * be_graph_duplicate (
	CuBook		* cubook
	)
{
	char		* oldname = cubook->book->prefs.active_graph,
			* newname;


	if ( ! oldname )
	{
		return NULL;
	}

	if ( ced_graph_duplicate ( cubook, oldname, &newname ) )
	{
		return NULL;
	}

	return newname;
}

int be_graph_selection_clip (
	CedSheet	* sheet,
	char		* buf,
	int		buflen
	)
{
	if ( ! sheet || ! sheet->book_tnode || ! sheet->book_tnode->key )
	{
		return 1;
	}

	int		r1,
			c1,
			r2,
			c2;
	char		sheetname [ buflen ],
			* smax,
			* dest,
			* src,
			cellrange [ buflen ];


	// Convert sheet name to valid utree string
	src = sheet->book_tnode->key;
	smax = sheetname + buflen - 3;

	for ( dest = sheetname; src[0] != 0 && dest < smax; dest ++ )
	{
		if (	src[0] == '\\' ||
			src[0] == '"'
			)
		{
			*dest++ = '\\';
		}

		dest[0] = *src++;
	}

	dest[0] = 0;

	ced_sheet_cursor_max_min ( sheet, &r1, &c1, &r2, &c2 );

	snprintf ( cellrange, buflen, "r%ic%i:r%ic%i", r1, c1, r2, c2 );
	cellrange [ buflen - 1 ] = 0;

	snprintf ( buf, buflen,
		"{ plot_graph_bar\n"
		"	fill_color=\"0x9999FF\"\n"
		"	gap=\"1\"\n"
		"	sheet=\"%s\"\n"
		"	data=\"%s\"\n"
		"	antialias=\"0\"\n"
		"}\n",
		sheetname, cellrange );

	buf [ buflen - 1 ] = 0;

	return 0;			// Success
}

