/*
	Copyright (C) 2013-2014 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#ifdef U_TK_QT4
	#include <mtqex4.h>
#endif

#ifdef U_TK_QT5
	#include <mtqex5.h>
#endif



#ifdef __cplusplus
	extern "C" {
#endif	//  __cplusplus


// C API

#include "be.h"



#ifdef __cplusplus
	}
#endif	//  __cplusplus



QT_BEGIN_NAMESPACE

class	QAction;
class	QCloseEvent;
class	QCheckBox;
class	QComboBox;
class	QDialog;
class	QEvent;
class	QFileDialog;
class	QGraphicsPixmapItem;
class	QGridLayout;
class	QKeyEvent;
class	QLabel;
class	QLineEdit;
class	QMenu;
class	QMenuBar;
class	QObject;
class	QScrollBar;
class	QSpinBox;
class	QString;
class	QStringList;
class	QTabWidget;
class	QTableWidget;
class	QTextEdit;
class	QWidget;
class	QWheelEvent;

QT_END_NAMESPACE



class	MainWindow;
class	CedView;
class	CedViewArea;
class	MyLineEdit;
class	MySaveFileDialog;
class	SortDialog;
class	SwatchDialog;



enum
{
	VIEW_ROW_HEADER		= 0,
	VIEW_ROW_PANE_TOP	= 1,
	VIEW_ROW_FREEZE		= 2,
	VIEW_ROW_PANE_BOTTOM	= 3,
	VIEW_ROW_SCROLL		= 4,

	VIEW_ROW_TOTAL		= 5
};

enum
{
	VIEW_COLUMN_HEADER	= 0,
	VIEW_COLUMN_PANE_LEFT	= 1,
	VIEW_COLUMN_FREEZE	= 2,
	VIEW_COLUMN_PANE_RIGHT	= 3,
	VIEW_COLUMN_SCROLL	= 4,

	VIEW_COLUMN_TOTAL	= 5
};

enum
{
	TAB_FIND		= 0,
	TAB_GRAPH		= 1,
	TAB_VIEW		= 2
};



#define BORDER_MENU_TOTAL	31



extern MainWindow	* mainwindow;



class MainWindow : public QWidget
{
	Q_OBJECT

public:
	MainWindow	( const char * prefs_filename = NULL );
	~MainWindow	();

	mtFont		* projectGetFont ();
	int		projectSetFont ( char * filename, int size );
			// 0 = success
			// 1 = failure with message box presented to user

	void		projectGetSheetGeometry ( int * r, int * c );
	CedSheet	* projectGetSheet ();	// Get from render
	CedSheet	* projectSetSheet ();	// Get from book, set render
	CedUI_file	* projectGetCedFile ();
	CedRender	* projectGetRender ();
	int		projectReportUpdates ( int error );

	void		insertCellText ( char * text );

	void		addFindRow (
				CedSheet * sheet,
				CedCell * cell,
				unsigned int r,
				unsigned int c
			);

	int		isEditingCellText ();
	int		isMainView ( CedView * v );

	void setCursorRange (
			int r1,
			int c1,
			int r2,
			int c2,
			int cursor_visible,
			int force_update,
			int follow_all	// Force cursor visibility everywhere
			);

	void updateChangesChores (
			int new_geometry,
			int block_sheet_recalcs
			);
	void		updateGraph ( char * graph_name );
	void		updateRecentFiles ();
	void		updateView ();
	void		updateViewConfig ();

public slots:
	void		pressGraphRedraw ();

protected:
	void		closeEvent ( QCloseEvent * event );
	void		keyPressEvent ( QKeyEvent * event );

private slots:
	void		graphChanged ( int i );
	void		findCellChanged (
				int currentRow,
				int currentColumn,
				int previousRow,
				int previousColumn
				);
	void		quicksumChanged ( int i );
	void		sheetChanged ( int i );

	void		pressUpdateCellRef ();
	void		pressStopCellRef ();
	void		pressTabCellRef ( int t );
	void		pressFocusOutCellRef ();
	void		pressUpdateCellText ();
	void		pressStopCellText ();
	void		pressTabCellText ( int t );
	void		pressArrowCellText ( int r );

	void		pressFileNew ();
	void		pressFileOpen ();
	void		pressFileImport ();
	void		pressFileSave ();
	void		pressFileSaveAs ();
	void		pressFileRecent ( int i );
	void		pressFileQuit ();

	void		pressEditUndo ();
	void		pressEditRedo ();
	void		pressEditCut ();
	void		pressEditCopy ();
	void		pressEditCopyVal ();
	void		pressEditCopyOutput ();
	void		pressEditTransformTrans ();
	void		pressEditTransformFlipH ();
	void		pressEditTransformFlipV ();
	void		pressEditTransformRotClock ();
	void		pressEditTransformRotAnti ();
	void		pressEditPaste ();
	void		pressEditPasteContent ();
	void		pressEditPastePrefs ();
	void		pressEditClear ();
	void		pressEditClearContent ();
	void		pressEditClearPrefs ();
	void		pressEditFixYears ();
	void		pressEditSelectAll ();

	void		pressSheetNew ();
	void		pressSheetDuplicate ();
	void		pressSheetRename ();
	void		pressSheetDelete ();
	void		pressSheetExport ();
	void		pressSheetExportOutput ();
	void		pressSheetFreezePanes ();
	void		pressSheetLock ();
	void		pressSheetPrevious ();
	void		pressSheetNext ();
	void		pressSheetRecalcBook ();
	void		pressSheetRecalc ();

	void		pressRowInsert ();
	void		pressRowInsertPasteHeight ();
	void		pressRowDelete ();
	void		pressRowSort ();

	void		pressColumnInsert ();
	void		pressColumnInsertPasteWidth ();
	void		pressColumnDelete ();
	void		pressColumnSort ();
	void		pressColumnSetWidth ();
	void		pressColumnSetWidthAuto ();

	void		pressOptionsFullScreen ();
	void		pressOptionsFind ();
	void		pressOptionsGraph ();
	void		pressOptionsView ();
	void		pressOptionsEditCell ();
	void		pressOptionsCellPrefs ();
	void		pressOptionsBookPrefs ();
	void		pressOptionsProgramPrefs ();
	void		pressOptionsBold ();
	void		pressOptionsBackgroundColor ();
	void		pressOptionsForegroundColor ();
	void		pressOptionsBorderColor ();
	void		pressOptionsBorder ( int i );
	void		pressOptionsHelp ();
	void		pressOptionsAbout ();

	void		pressFind ();
	void		pressFindWildcards ();
	void		pressFindCase ();
	void		pressFindValue ();
	void		pressFindSheets ();

	void		pressGraphNew ();
	void		pressGraphDuplicate ();
	void		pressGraphRename ();
	void		pressGraphDelete ();
	void		pressGraphExport ();
	void		pressGraphSClipboard ();

private:
	void		coreRecalcBook ();
	void		coreRecalcSheet ();

	int		clipboardClearSelection ( int mode );
	void		clipboardCopyRouter ( int mode );
	int		clipboardCopySelection ( int mode );
	void		clipboardFlushInternal ();
	int		clipboardGetMtcelledit ( QClipboard * clipboard );
	int		clipboardObtainPaste ();
	int		clipboardPasteAtCursor ( int mode );
	int		clipboardReadSystem ();
	void		clipboardSetOwner ();
	void		clipboardTransform ( int mode );

	void		createMenus ();
	void		createQuicksum ();

	QStringList	getFileExportTypes ();
	int		getSelectionPosition (	// Get position & obtain paste
				CedSheet * sheet,	// All args optional
				int * row,
				int * col
				);
				// 0 = Success

	void		menuInit (
				QAction		** action,
				const char	* text,
				const char	* shortcut,
				const char	* icon
				);

	int		okToLoseChanges ();
			// 0 = Changes and user doesn't consent to losing them
			// 1 = No changes or user consents to losing changes

	void		projectClearAll ();
	void		projectGraphStoreChanges ();
	void		projectGraphRedraw ();

			// 0 = success
			// 1 = failure with message box presented to user
	int		projectLoad ( char * filename );
	int		projectSave ( char * filename, CedFileType format );
	int		projectImport ( char * filename );

	void		projectRegister ( char * filename, int change );
	int		projectRenameSheet (
				CedSheet * sheet,
				QString new_name
				);
	int		projectRenameGraph ( QString newName );
	void		projectSetSheetGeometry ();

	void		reportLargeTSV ();

	void		setCellFromInput ( int rowd, int cold );

	void		updateEntryCellref ();
	void		updateEntryCelltext ();
	void		updateMainArea ();
	void		updateMenus ();
	void		updateQuicksumLabel ();
	void		updateRecalcBook ();
	void		updateRecalcSheet ();
	void		updateRender ();
	void		updateSheetSelector ();
	void		updateTitleBar ();

	QAction
			* actFileRecentSeparator,
			* actFileRecent [ RECENT_MENU_TOTAL ],
			* actFileQuit,

			* actEditUndo,
			* actEditRedo,

			* actSheetFreezePanes,
			* actSheetLock,

			* actFindWildcards,
			* actFindCase,
			* actFindValue,
			* actFindSheets,

			* actGraphDuplicate,
			* actGraphRename,
			* actGraphDelete,
			* actGraphRedraw,
			* actGraphExport,
			* actGraphSClipboard
			;

	QWidget		* rightSplit;
	QTabWidget	* tabWidget;
	QMenuBar
			* findMenuBar,
			* graphMenuBar,
			* menuBar
			;
	QTextEdit	* graphTextEdit;

	qexButtonMenu	* buttonGraph,
			* buttonQuicksum,
			* buttonSheet;

	QLineEdit	* editCellref,
			* editCelltext,
			* editFindText
			;
	QTableWidget	* findTable;
	QLabel		* labelQuicksum;

	qexImage	* graphWidget;

	CedView		* viewMain,
			* viewTab;

	CedUI_file	* cedFile;
	CedUI_clip	* cedClipboard;
	CedRender	render;

	unsigned int	sheetRows,
			sheetCols;

	int		memChanged,
			lastExportSheetType,
			lastExportGraphType
			;
};

class CedView : public QWidget
{
	Q_OBJECT

public:
	CedView		();

	void		ensureVisible ( CedSheet * sheet, int row, int column );

	int		getBell ();
	int		getHScrollValue ();
	int		getVScrollValue ();
	int		getVisibleRows ( CedRender * render = NULL );
	int		getPageColumnsLeft ( int column );
	int		getPageColumnsRight ( int column );

	bool		hasFocus ();

	void		reconfigure ();	// Update pane view, scrollbars etc.

	void		redrawArea ( // Update all visible areas of this sheet
				CedSheet * sheet,
				int nr1, int nc1,
				int nr2, int nc2
				);

	void		updateGeometry ();
	void		updateRedraw ();

	void		setBell ();
	void		setFocus ();
	void		setScrollbars ();

private slots:
	void		vscrollAction ( int action );
	void		vscrollMove ( int value );

	void		hscrollAction ( int action );
	void		hscrollMove ( int value );

private:
	void		setPanes ();
	void		setRowHeaderWidth ();

	void		wheelEvent ( QWheelEvent * event );



	CedViewArea	* area [ CEDVIEW_AREA_TOTAL ];

	QGridLayout	* layout;

	QScrollBar	* hscroll,
			* vscroll;

	int		bellState[2],
			hscrollLast,
			vscrollLast
			;
};

class CedViewArea : public QWidget
{
	Q_OBJECT

public:
	CedViewArea	( CedView * cv, int id );

protected:
	bool		event ( QEvent * event );	// Catch tabs

private:
	void		paintEvent ( QPaintEvent * event );

	void		mousePressEvent ( QMouseEvent * event );
	void		mouseMoveEvent ( QMouseEvent * event );
	void		mouseEventRouter ( QMouseEvent * event, int caller );

	CedView		* cedview;
	int		areaID;
};

class MyLineEdit : public QLineEdit
{
	Q_OBJECT

signals:
	void		pressUpdateKey ();	// Return or Enter keys
	void		pressStopKey ();	// Escape
	void		pressTabKey ( int t );	// Tab = 1 Ctrl+Tab = -1
	void		pressArrowKey ( int r );// Up = -1, down = 1
	void		pressFocusOut ();

protected:
	bool		event ( QEvent * event );
	void		focusOutEvent ( QFocusEvent * e );
};


class MySaveFileDialog : public QFileDialog
{
	Q_OBJECT

public:
	MySaveFileDialog (
			QWidget		* parent = NULL,
			QString		title = QString (),
			QStringList	formatList = QStringList (),
			int		format = 0,
			char		* filename = NULL
			);

	int		getFormat ();

private:
	QComboBox	* comboFormat;
};

class SortDialog : public QDialog
{
	Q_OBJECT

public:
	SortDialog (	int axis,		 // 0 = Rows, 1 = Columns
			QWidget * parent = NULL
			);

private slots:
	void		pressClose ();
	void		pressMoveDown ();
	void		pressMoveUp ();
	void		pressSort ();

	void		spinChanged ( int i );
	void		directionChanged ( int i );
	void		caseChanged ( int i );
	void		tableRowClick (
				int currentRow,
				int currentColumn,
				int previousRow,
				int previousColumn
				);

private:
	QTableWidget	* tableWidget;
	QSpinBox	* spinBoxWidget;
	QCheckBox	* checkDirection,
			* checkCase;

	void		setRowValues ( int row );
	int		getTableRow ();
	void		swapRows ( int delta );
			// Swap table rows, update widget, cursor to new row
};

class SwatchDialog : public QDialog
{
	Q_OBJECT

public:
	SwatchDialog	( QString title );

	int		getColor ();

private slots:
	void		pressButton ( int i );
	void		pressCancel ();

private:
	int		color;
};

