/*
	Copyright (C) 2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "tk_qt4.h"



void MainWindow :: pressFileNew ()
{
	if ( okToLoseChanges () )
	{
		projectClearAll ();
	}
}

void MainWindow :: pressFileOpen ()
{
	if ( okToLoseChanges () )
	{
		QString filename = QFileDialog :: getOpenFileName ( this,
			tr ( "Load Project File" ),
			mtQEX::qstringFromC (
				prefs_get_string ( CED_INIFILE_LAST_DIR ) ),
			NULL, NULL, QFileDialog::DontUseNativeDialog );


		if ( ! filename.isEmpty () )
		{
			projectLoad ( filename.toUtf8 ().data () );
		}
	}
}

void MainWindow :: pressFileImport ()
{
	QString filename = QFileDialog :: getOpenFileName ( this,
		tr ( "Import Project File" ),
		mtQEX::qstringFromC (
			prefs_get_string ( CED_INIFILE_LAST_DIR ) ),
		NULL, NULL, QFileDialog::DontUseNativeDialog );


	if ( ! filename.isEmpty () )
	{
		projectImport ( filename.toUtf8 ().data () );
	}
}

void MainWindow :: pressFileSave ()
{
	if ( ! cedFile->name || 0 == cedFile->name[0] )
	{
		pressFileSaveAs ();
	}
	else
	{
		projectSave ( cedFile->name, cedFile->type );
	}
}

void MainWindow :: pressFileSaveAs ()
{
	QStringList	list = getFileExportTypes ();
	MySaveFileDialog dialog ( this, tr ( "Save Project File" ), list,
				cedFile->type - 1, cedFile->name );

	if ( dialog.exec () )
	{
		QStringList	fileList = dialog.selectedFiles ();
		QString		filename = fileList.at ( 0 );
		int		format = dialog.getFormat ();


		if ( ! filename.isEmpty () )
		{
			projectSave ( filename.toUtf8 ().data (),
				(CedFileType) (format + 1) );
		}
	}
}

void MainWindow :: pressFileRecent ( int i )
{
	if ( okToLoseChanges () )
	{
		projectLoad ( be_get_recent_filename ( i ) );
	}
}

void MainWindow :: pressFileQuit ()
{
	close ();
}

void MainWindow :: projectClearAll ()
{
	cuf_file_book_new ( cedFile );
	projectSetSheet ();		// Old sheet stale so update renderer
	updateMenus ();

	memChanged = 0;

	updateSheetSelector ();
	updateTitleBar ();

	setCursorRange ( 1, 1, 1, 1, 1, 1, 1 );

	updateGraph ( NULL );
}

int MainWindow :: projectLoad (
	char		* filename
	)
{
	CedSheet	* sheet;
	CedSheetPrefs	* tmp_sheet_prefs = NULL;


	sheet = cuf_file_get_sheet ( cedFile );
	if ( sheet )
	{
		if ( ced_file_type_class ( cedFile->type ) == 1 )
		{
			if ( prefs_get_int (
				CED_INIFILE_SHEET_PREFS_PERSIST ) )
			{
				tmp_sheet_prefs = ced_sheet_prefs_new ();
				ced_sheet_prefs_copy ( tmp_sheet_prefs,
					&sheet->prefs );
			}
		}
	}

	if ( cuf_file_load ( cedFile, filename ) )
	{
		QMessageBox :: critical ( this, tr ( "Error" ),
			tr ("Unable to load project file: %1").
			arg ( mtQEX::qstringFromC ( filename ) ) );

		ced_sheet_prefs_free ( tmp_sheet_prefs );

		return 1;
	}

	projectSetSheet ();		// Old sheet stale so update renderer

	if ( tmp_sheet_prefs )
	{
		if ( ced_file_type_class ( cedFile->type ) == 1 )
		{
			sheet = projectGetSheet ();
			if ( sheet )
			{
				ced_sheet_prefs_copy ( &sheet->prefs,
					tmp_sheet_prefs );
			}
		}

		ced_sheet_prefs_free ( tmp_sheet_prefs );
		tmp_sheet_prefs = NULL;
	}

	updateSheetSelector ();
	updateGraph ( cedFile->cubook->book->prefs.active_graph );

	if (	cedFile->cubook->book->prefs.auto_recalc ||
		! ( cedFile->type == CED_FILE_TYPE_TSV_VAL_BOOK ||
			cedFile->type == CED_FILE_TYPE_LEDGER_VAL_BOOK )
		)
	{
		// Books with values don't need recalculating
		updateRecalcBook ();
	}

	updateMenus ();

	memChanged = 0;

	projectRegister ( cedFile->name, 1 );

	if ( cedFile->cubook->book->prefs.disable_locks )
	{
		QMessageBox :: warning ( this, tr ( "Warning" ),
			tr ("This book has the disable_locks flag set, "
			"which is potentially dangerous." ) );
	}

	return 0;
}

void MainWindow :: projectRegister (
	char		* filename,
	int		change
	)
{
	if ( be_register_project ( cedFile, filename, change ) )
	{
		return;
	}

	updateTitleBar ();
	updateRecentFiles ();
}

MySaveFileDialog :: MySaveFileDialog (
	QWidget		* parent,
	QString		title,
	QStringList	formatList,
	int		format,
	char		* filename
	)
	:
	QFileDialog	( parent ),
	comboFormat	()
{
	QString		s;
	int		i;


	setWindowTitle ( title );
	setAcceptMode ( QFileDialog::AcceptSave );
	setOptions ( QFileDialog::DontUseNativeDialog );

	if ( ! filename || 0 == filename[0] )
	{
		filename = prefs_get_string ( CED_INIFILE_LAST_DIR );
		setDirectory ( mtQEX::qstringFromC ( filename ) );
	}
	else
	{
		selectFile ( mtQEX::qstringFromC ( filename ) );
	}

	if ( formatList.count () > 0 )
	{
		QLayout		* l = layout ();
		QGridLayout	* grid = dynamic_cast<QGridLayout *>( l );
		QWidget		* w;
		QHBoxLayout	* row;



		w = new QWidget ();

		if ( grid )
		{
			grid->addWidget ( w, grid->rowCount (), 0, 1,
				grid->columnCount () );
		}
		else
		{
			// If all else fails just add it!
			l->addWidget ( w );
		}

		row = new QHBoxLayout;
		row->setMargin ( 0 );
		w->setLayout ( row );

		QLabel * label = new QLabel ( tr ( "File Format:" ) );
		row->addWidget ( label );

		comboFormat = new QComboBox;
		row->addWidget ( comboFormat );

		w = new QWidget;
		w->setSizePolicy ( QSizePolicy ( QSizePolicy::MinimumExpanding,
			QSizePolicy::Preferred ) );
		row->addWidget ( w );

		for ( i = 0; i < formatList.count (); i++ )
		{
			s = formatList.at ( i );

			comboFormat->addItem ( s );
		}

		comboFormat->setCurrentIndex ( format );
	}
}

int MySaveFileDialog :: getFormat ()
{
	if ( comboFormat )
	{
		return comboFormat->currentIndex ();
	}

	return 0;
}

void MainWindow :: reportLargeTSV ()
{
	QMessageBox :: critical ( this, tr ( "Error" ),
		tr ("The sheet geometry is too large to save in this file"
		" format." ) );
}

int MainWindow :: projectSave ( char * filename, CedFileType format )
{
	int		res;


	if (	format == CED_FILE_TYPE_TSV_BOOK	||
		format == CED_FILE_TYPE_TSV_VAL_BOOK	||
		format == CED_FILE_TYPE_LEDGER_BOOK	||
		format == CED_FILE_TYPE_LEDGER_VAL_BOOK
		)
	{
		projectGraphStoreChanges ();
	}

	res = cuf_file_save ( cedFile, filename, format );
	if ( res == 1 )
	{
		reportLargeTSV ();

		return 1;
	}

	if ( res )
	{
		QMessageBox :: critical ( this, tr ( "Error" ),
			tr ("Unable to save file: %1").
			arg ( mtQEX::qstringFromC ( filename ) ) );

		return 1;
	}

	memChanged = 0;
	projectRegister ( filename, 1 );

	return 0;
}

int MainWindow :: projectImport ( char * filename )
{
	CedUI_file	* uifile;
	int		sheet_tot,
			sheet_fail,
			file_tot,
			file_fail,
			res;
	char		buf [ 2048 ];


	uifile = cuf_file_new ();
	if ( ! uifile )
	{
		QMessageBox :: critical ( this, tr ( "Error" ),
			tr ("Unable to allocate memory to import file: %1").
			arg ( mtQEX::qstringFromC ( filename ) ) );

		return 1;
	}

	if (	cuf_file_load ( uifile, filename )	||
		! uifile->cubook			||
		! uifile->cubook->book
		)
	{
		cuf_file_free ( uifile );

		QMessageBox :: critical ( this, tr ( "Error" ),
			tr ("Unable to import file: %1").
			arg ( mtQEX::qstringFromC ( filename ) ) );

		return 1;
	}

	res = cuf_book_merge ( cedFile->cubook, uifile->cubook->book,
		&sheet_tot, &sheet_fail, &file_tot, &file_fail );

	cuf_file_free ( uifile );

	projectReportUpdates ( res );

	updateSheetSelector ();		// Done here as need for success/fail

	// Update graph selector
	updateGraph ( cedFile->cubook->book->prefs.active_graph );

	snprintf ( buf, sizeof ( buf ),
		"%i sheets imported.\n"
		"%i sheets not imported due to identical names.\n"
		"%i graphs/files imported.\n"
		"%i graphs/files not imported due to identical names.",
		sheet_tot, sheet_fail, file_tot, file_fail );

	buf [ sizeof ( buf ) - 1 ] = 0;

	QMessageBox :: information ( this, tr ( "Information" ),
		mtQEX::qstringFromC ( buf ) );

	if (	res == CU_ERROR_LOCKED_CELL	||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return 1;
	}

	if ( sheet_tot > 0 )
	{
		be_update_file_to_book ( cedFile );
	}

	updateChangesChores ( 0, 1 );

	return 0;			// Success
}

