/*
	Copyright (C) 2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "be.h"



int be_font_new (
	char		* filename,
	int		size,
	mtFont		** fontDest,
	int		* fontw,
	int		* fonth
	)
{
	mtFont		* newfont;
	mtFontRender	* ren;


	if ( filename && filename[0] == 0 )
	{
		filename = NULL;
	}

	newfont = mtpixel_font_new ( filename, filename ? 0 : size );
	if ( ! newfont )
	{
		return 1;
	}

	if ( fontDest[0] )
	{
		mtpixel_font_destroy ( fontDest[0] );
	}

	fontDest[0] = newfont;

	if ( filename )
	{
		mtpixel_font_set_size ( newfont, size );

		prefs_set_string ( PREFS_FONT_FILENAME, filename );
	}
	else
	{
		prefs_set_string ( PREFS_FONT_FILENAME, "" );
	}

	prefs_set_int ( PREFS_FONT_SIZE, size );

	ren = mtpixel_font_render_new ( newfont, "01234567890", NULL,
		MTPIXEL_IMAGE_RGB );

	if ( ren )
	{
		if ( ! mtpixel_font_extent ( ren ) )
		{
			fontw[0] = (ren->width + 5) / 11;
			fonth[0] = ren->height;
		}

		mtpixel_font_render_destroy ( ren );
	}

	return 0;			// Success
}

typedef struct
{
	int		last_seat;
	mtImage		* image;
	mtBrush		* brush;
} uptabSTATE;



static int uptab_cb (
	int		seat,
	mtBrush		** brush_fill,
	mtBrush		** brush_outline,
	void		* user_data
	)
{
	uptabSTATE	* state = (uptabSTATE *)user_data;


	if ( state->image->channel[MTPIXEL_CHANNEL_IMAGE][0] != 255 )
	{
		// Terminate if the last rendering found a seat

		return 2;
	}

	state->last_seat = seat;

	brush_fill[0] = state->brush;
	brush_outline[0] = state->brush;

	return 0;			// Render & continue
}

int be_get_seat (
	int		x,
	int		y,
	mtTree		* polymap_tree,
	double		* table,
	double		map_zoom
	)
{
	uptabSTATE	state = { -1 };


	state.image = mtpixel_image_new_rgb ( 1, 1 );
	if ( ! state.image )
	{
		return -1;
	}

	state.image->channel[MTPIXEL_CHANNEL_IMAGE][0] = 255;

	state.brush = mtpixel_brush_new ();
	if ( ! state.brush )
	{
		mtpixel_image_destroy ( state.image );

		return -1;
	}

	mtpixel_brush_set_RGB ( state.brush, 0, 0 );
	mtpixel_brush_set_size ( state.brush, 0 );

	eleana_poly_render_custom ( state.image, x, y,
		polymap_tree, table, map_zoom, uptab_cb, &state );

	if ( state.image->channel[MTPIXEL_CHANNEL_IMAGE][0] == 255 )
	{
		// No seat was found

		state.last_seat = -1;
	}

	mtpixel_image_destroy ( state.image );
	mtpixel_brush_destroy ( state.brush );

	return state.last_seat;
}

void be_file_to_last_dir ( char * filename )
{
	char		buf [ 2048 ],
			* tp;


	snprintf ( buf, sizeof( buf ), "%s", filename );
	buf [ sizeof ( buf ) - 1 ] = 0;

	tp = strrchr ( buf, MTKIT_DIR_SEP );

	if ( tp )
	{
		tp [ 0 ] = 0;
	}

	prefs_set_string ( PREFS_LAST_DIR, buf );
}

int be_save_polymap (
	char		* filename,
	mtTree		* polymap_tree,
	double		* table,
	double		zoom
	)
{
	const char	* s[5] = {
				PREFS_MAP_COL_A,
				PREFS_MAP_COL_B,
				PREFS_MAP_COL_C,
				PREFS_MAP_COL_D,
				PREFS_MAP_COL_INK
				};
	int		cols[5] = {0},
			i;
	mtImage		* image = NULL;


	image = mtpixel_image_new_rgb ( POLYMAP_WIDTH * zoom + 1,
		POLYMAP_HEIGHT * zoom + 1 );

	if ( ! image )
	{
		return 1;
	}

	for ( i = 0; i < 5; i++ )
	{
		cols[i] = prefs_get_int ( s[i] );
	}

	memset ( image->channel [ MTPIXEL_CHANNEL_IMAGE ], 255,
		image->width * image->height * 3 );
	eleana_poly_render ( image, 0, 0, polymap_tree, table, zoom, cols );

	if ( mtpixel_image_save ( image, filename, MTPIXEL_FILE_TYPE_PNG, 5 ) )
	{
		mtpixel_image_destroy ( image );

		return 1;
	}

	mtpixel_image_destroy ( image );

	return 0;
}

