/*
	Copyright (C) 2011-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "be.h"



static mtPrefs		* prefs_mem;
static char		prefs_filename [ 2048 ];



void prefs_init ( void )
{
	mtPrefItem	prefs_table[] = {

{ PREFS_LAST_DIR,	MTKIT_PREF_TYPE_STR },

{ PREFS_LAST_YEAR,	MTKIT_PREF_TYPE_INT, "0" },

{ PREFS_WINDOW_X, MTKIT_PREF_TYPE_INT, "50" },
{ PREFS_WINDOW_Y, MTKIT_PREF_TYPE_INT, "50" },
{ PREFS_WINDOW_W, MTKIT_PREF_TYPE_INT, "500" },
{ PREFS_WINDOW_H, MTKIT_PREF_TYPE_INT, "500" },

{ PREFS_GRAPH_COL_A, MTKIT_PREF_TYPE_RGB, "255" },
{ PREFS_GRAPH_COL_B, MTKIT_PREF_TYPE_RGB, "16711680" },
{ PREFS_GRAPH_COL_C, MTKIT_PREF_TYPE_RGB, "16776960" },
{ PREFS_GRAPH_COL_D, MTKIT_PREF_TYPE_RGB, "11842740" },
{ PREFS_GRAPH_COL_INK, MTKIT_PREF_TYPE_RGB, "0" },
{ PREFS_GRAPH_COL_PAGE, MTKIT_PREF_TYPE_RGB, "16777215" },
{ PREFS_GRAPH_COL_G1, MTKIT_PREF_TYPE_RGB, "14474460" },
{ PREFS_GRAPH_COL_G2, MTKIT_PREF_TYPE_RGB, "13158600" },

{ PREFS_MAP_COL_A, MTKIT_PREF_TYPE_RGB, "3302655", NULL, map_redraw_ini },
{ PREFS_MAP_COL_B, MTKIT_PREF_TYPE_RGB, "16711680", NULL, map_redraw_ini },
{ PREFS_MAP_COL_C, MTKIT_PREF_TYPE_RGB, "16776960", NULL, map_redraw_ini },
{ PREFS_MAP_COL_D, MTKIT_PREF_TYPE_RGB, "11842740", NULL, map_redraw_ini },
{ PREFS_MAP_COL_INK, MTKIT_PREF_TYPE_RGB, "0", NULL, map_redraw_ini },

{ PREFS_FONT_FILENAME, MTKIT_PREF_TYPE_FILE },
{ PREFS_FONT_SIZE, MTKIT_PREF_TYPE_INT, "16" },

{ NULL }
		};


	prefs_mem = mtkit_prefs_new ( prefs_table );

	snprintf ( prefs_filename, sizeof ( prefs_filename ), "%s/.%s",
		mtkit_file_home (), BIN_NAME );
	prefs_filename [ sizeof ( prefs_filename ) - 1 ] = 0;
}

void prefs_load ( void )
{
	mtkit_prefs_load ( prefs_mem, prefs_filename );
}

void prefs_save ( void )
{
	mtkit_prefs_save ( prefs_mem, prefs_filename );
}

void prefs_close ( void )
{
	mtkit_prefs_destroy ( prefs_mem );
	prefs_mem = NULL;
}

mtPrefs * prefs_file ( void )
{
	return prefs_mem;
}

void prefs_set_int (
	const char	* key,
	int		value
	)
{
	mtkit_prefs_set_int ( prefs_mem, (char *)key, value );
}

void prefs_set_string (
	const char	* key,
	const char	* value
	)
{
	mtkit_prefs_set_str ( prefs_mem, (char *)key, (char *)value );
}

void prefs_set_double (
	const char	* key,
	double		value
	)
{
	mtkit_prefs_set_double ( prefs_mem, (char *)key, value );
}

int prefs_get_int (
	const char	* key
	)
{
	int		res = 0;


	mtkit_prefs_get_int ( prefs_mem, (char *)key, &res );

	return res;
}

char * prefs_get_string (
	const char	* key
	)
{
	static char	* nothing = "";
	char		* res = nothing;


	mtkit_prefs_get_str ( prefs_mem, (char *)key, &res );

	if ( ! res )
	{
		return nothing;
	}

	return res;
}

double prefs_get_double (
	const char	* key
	)
{
	double		res = 0;


	mtkit_prefs_get_double ( prefs_mem, (char *)key, &res );

	return res;
}
