/*
	Copyright (C) 2008-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <unistd.h>
#include <time.h>
#include <math.h>

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include <mtgex.h>

#include "be.h"



#ifdef U_NLS
	#include <libintl.h>
	#define _(text) gettext(text)
	#define __(text) gettext(text)
#else
	#define _(text) text
	#define __(text) text
#endif



#define _GW_LABEL_RES_COL_V	0
#define _GW_LABEL_RES_COL_VP	1
#define _GW_LABEL_RES_COL_VPS	2
#define _GW_LABEL_RES_COL_S	3
#define _GW_LABEL_RES_COL_SC	4
#define _GW_LABEL_RES_COLS	5

// row = 0..seats
// col = as EL_TAB_*
#define EL_TAB_VALUE( row, col )	global.table[EL_TAB_TOTAL * row + col]

#define FS_SELECT_FILE		15
#define FS_SELECT_DIR		16



enum
{
	GW_SDRAW_MAP,
	GW_SDRAW_AREA_MAP,

	GW_GIMAGE_GRAPH,

	GW_SPINSLIDER_SWING_AB,
	GW_SPINSLIDER_SWING_AC,
	GW_SPINSLIDER_SWING_AO,
	GW_SPINSLIDER_SWING_BC,
	GW_SPINSLIDER_SWING_BO,
	GW_SPINSLIDER_SWING_CO,

	GW_SPINSLIDER_MAPZOOM,

	GW_BMENU_PICK_YEAR,

	GW_LABEL_SWING_HEAD_AB,
	GW_LABEL_SWING_HEAD_AC,
	GW_LABEL_SWING_HEAD_AO,
	GW_LABEL_SWING_HEAD_BC,
	GW_LABEL_SWING_HEAD_BO,
	GW_LABEL_SWING_HEAD_CO,

	GW_LABEL_RESULTS_HEAD_A,
	GW_LABEL_RESULTS_HEAD_B,
	GW_LABEL_RESULTS_HEAD_C,
	GW_LABEL_RESULTS_HEAD_O,
	GW_LABEL_RESULTS_HEAD_TOTAL,

	GW_LABEL_RES_A_V,
	GW_LABEL_RES_A_VP,
	GW_LABEL_RES_A_VPS,
	GW_LABEL_RES_A_S,
	GW_LABEL_RES_A_SC,

	GW_LABEL_RES_B_V,
	GW_LABEL_RES_B_VP,
	GW_LABEL_RES_B_VPS,
	GW_LABEL_RES_B_S,
	GW_LABEL_RES_B_SC,

	GW_LABEL_RES_C_V,
	GW_LABEL_RES_C_VP,
	GW_LABEL_RES_C_VPS,
	GW_LABEL_RES_C_S,
	GW_LABEL_RES_C_SC,

	GW_LABEL_RES_O_V,
	GW_LABEL_RES_O_VP,
	GW_LABEL_RES_O_VPS,
	GW_LABEL_RES_O_S,
	GW_LABEL_RES_O_SC,

	GW_LABEL_RES_T_V,
	GW_LABEL_RES_T_VP,
	GW_LABEL_RES_T_VPS,
	GW_LABEL_RES_T_S,
	GW_LABEL_RES_T_SC,

	GW_TOTAL
};

enum	// File selector quarks
{
	FS_XTRA_HBOX		= 1,
	FS_SPIN_FONT_SIZE
};

enum
{
	FS_LOAD_PROJECT		= 1,
	FS_SAVE_GRAPH,
	FS_SAVE_POLYMAP,
	FS_PICK_FONT,
};



typedef struct
{
	mtFont		* font;
	int		font_size,
			font_w,
			font_h;

	char		mem_filename [ 2048 ],
			party_name [ 4 ][ PART_NAME_SIZE ]
			;
	GtkWidget	* main_window,
			* gw [ GW_TOTAL ],
			* cursor_left_table,
			* notebook_left
			;

	gexImage	* gimage_carto,
			* gimage_diagram;

	double		swing [ EL_SWING_TOTAL ],
			map_zoom;
	mtImage		* diagram_image;
	int		seats,
			party_seats [ 4 ],
			eleana_swing,
			carto_zoom,
			diag_zoom;

	eleanaINDEX	* eleana_index;
	CedSheet	* sheet_full;
	double		* table;
	mtTree		* polymap_tree,	// Tree containing all of the map
					// polygons
			*seat_names;	// List of seat names indexed by number
					// 0..(seats-1)
	CedIndex	*seat_index;	// Index of seat coords by name
} GUI_Global;



GUI_Global	global;



int project_load (
	char * filename
	);

int project_save_graph (
	char * filename
	);

int project_save_polymap (
	char * filename
	);

int project_set_font (
	char * filename,
	int size
	);

void pressed_help (
	GtkMenuItem	* menu_item,
	gpointer	user_data,
	gint		item
	);

GtkTextBuffer * swing_report_window_init (
	char		* txt
	);

void file_selector (
	int		action_type,
	GtkWidget	* main_window,
	mtPrefs		* inifile,
	char		* filename
	);

