/*
	Copyright (C) 2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "tk_qt4.h"



MapArea :: MapArea ()
	:
	oldMouseX	( 0 ),
	oldMouseY	( 0 )
{
	setAttribute ( Qt::WA_OpaquePaintEvent );
	setAttribute ( Qt::WA_NoSystemBackground );
}

mtImage * MainWindow :: prepRender (
	int		px,
	int		py,
	int		pw,
	int		ph
	)
{
	mtImage		* image;


	if ( pw < 1 || ph < 1 )
	{
		return NULL;
	}

	image = mtpixel_image_new_rgb ( pw, ph );
	if ( ! image )
	{
		return NULL;
	}

	if ( yPolymapTree )
	{
		const char	* s[5] = {
					PREFS_MAP_COL_A,
					PREFS_MAP_COL_B,
					PREFS_MAP_COL_C,
					PREFS_MAP_COL_D,
					PREFS_MAP_COL_INK
					};
		int		cols[5] = {0},
				i,
				w,
				h;


		w = polymapScrollArea->viewport ()->width ();
		h = polymapScrollArea->viewport ()->height ();

		for ( i = 0; i < 5; i++ )
		{
			cols[i] = prefs_get_int ( s[i] );
		}

		memset ( image->channel[MTPIXEL_CHANNEL_IMAGE], 255,
			pw * ph * 3 );

		eleana_poly_render ( image, px - w, py - h,
			yPolymapTree, yTable, zoomMap, cols );
	}

	return image;
}

void MapArea :: paintEvent (
	QPaintEvent	* event
	)
{
	mtImage		* image;
	int		px,
			py,
			pw,
			ph
			;


	px = event->rect ().x ();
	py = event->rect ().y ();
	pw = event->rect ().width ();
	ph = event->rect ().height ();

	image = mainwindow->prepRender ( px, py, pw, ph );
	if ( ! image )
	{
		return;
	}

	QImage * im = new QImage (
		(const uchar *)image->channel[MTPIXEL_CHANNEL_IMAGE],
		pw, ph, pw * 3, QImage::Format_RGB888 );

	QPainter p ( this );
	p.drawImage ( QPoint ( px, py ), im[0] );

	delete im;		// Delete before rgb image destroy

	mtpixel_image_destroy ( image );
}

void MainWindow :: clickMap ( int x, int y )
{
	int		i,
			candidates,
			votes,
			seatnum;
	char		buf[1024],
			* st_list[10] = { NULL };
	mtTreeNode	* node;
	CedIndexItem	* index_item;
	CedCell		* cell;
	QLabel		* label;


	if ( ! ySheetFull )
	{
		// No data loaded

		return;
	}

	x -= polymapScrollArea->viewport ()->width ();
	y -= polymapScrollArea->viewport ()->height ();

	clearSeatTable ();

	seatnum = be_get_seat ( x, y, yPolymapTree, yTable, zoomMap );

	if ( seatnum < 0 )
	{
		return;
	}

	node = mtkit_tree_node_find ( ySeatNames,
		(void *)(intptr_t)seatnum );

	if ( ! node )
	{
		return;
	}

	st_list[0] = (char *)node->data;
	votes = 0;

	if ( ced_index_query ( ySeatIndex, 0, st_list[0], &index_item ) != 1 ||
		! index_item
		)
	{
		return;
	}

	// Count candidates and the total votes
	for ( candidates = 0; ; candidates ++ )
	{
		cell = ced_sheet_get_cell ( ySheetFull,
			index_item->row + candidates, FULL_COL_VOTES );

		if ( ! cell )
		{
			break;
		}

		votes += cell->value;
	}

	cell = ced_sheet_get_cell ( ySheetFull, index_item->row,
		FULL_COL_COUNTY );

	if ( cell && cell->text )
	{
		st_list[1] = (char *)" (";
		st_list[2] = cell->text;
		st_list[3] = (char *)")";
	}

	st_list[9] = mtkit_strjoinv ( NULL, st_list );
	if ( ! st_list[9] )
	{
		return;
	}

	label = new QLabel ( mtQEX::qstringFromC ( st_list[9] ) );
	gridSeat->addWidget ( label, 0, 0, 1, 4 );

	free ( st_list[9] );
	st_list[9] = NULL;

	sprintf ( buf, "%i", votes );

	label = new QLabel ( " " );
	gridSeat->addWidget ( label, 2, 0 );

	label = new QLabel ( " " );
	gridSeat->addWidget ( label, candidates + 3, 0 );

	label = new QLabel ( " " );
	gridSeat->addWidget ( label, candidates + 5, 0 );

	label = new QLabel ( tr ( "Turnout" ) );
	gridSeat->addWidget ( label, candidates + 4, 1 );

	label = new QLabel ( buf );
	gridSeat->addWidget ( label, candidates + 4, 2 );

	label = new QLabel ( tr ( "Electorate" ) );
	gridSeat->addWidget ( label, candidates + 6, 1 );

	cell = ced_sheet_get_cell ( ySheetFull, index_item->row,
		FULL_COL_ELECTORATE );

	if ( cell && cell->text )
	{
		label = new QLabel ( mtQEX::qstringFromC ( cell->text ) );
		gridSeat->addWidget ( label, candidates + 6, 2 );

		sprintf ( buf, "%.1f%%", 100 * ( (double)votes ) /
			cell->value );

		label = new QLabel ( buf );
		gridSeat->addWidget ( label, candidates + 4, 3 );
	}

	// Populate table with candidates
	for ( i = 0; i < candidates; i++ )
	{
		cell = ced_sheet_get_cell ( ySheetFull,
			index_item->row + i, FULL_COL_MP_NAME );

		if ( cell )
		{
			label = new QLabel ( mtQEX::qstringFromC (
				cell->text ) );
			gridSeat->addWidget ( label, i + 3, 0 );
		}

		cell = ced_sheet_get_cell ( ySheetFull,
			index_item->row + i, FULL_COL_PARTY );

		if ( cell )
		{
			label = new QLabel ( mtQEX::qstringFromC (
				cell->text ) );
			gridSeat->addWidget ( label, i + 3, 1 );
		}

		cell = ced_sheet_get_cell ( ySheetFull,
			index_item->row + i, FULL_COL_VOTES );

		if ( cell )
		{
			label = new QLabel ( mtQEX::qstringFromC (
				cell->text ) );
			gridSeat->addWidget ( label, i + 3, 2 );

			sprintf ( buf, "%.1f%%",
				100 * cell->value / ( (double)votes ) );

			label = new QLabel ( buf );
			gridSeat->addWidget ( label, i + 3, 3 );
		}
	}
}

void MapArea :: mouseEventRouter (
	QMouseEvent	* event,
	int		caller	// 0 = Press 2 = Move
	)
{
	int		x,
			y
			;


	if (	! ( event->buttons () & Qt::LeftButton ) &&
		! ( event->buttons () & Qt::RightButton )
		)
	{
		// We only look at left/right clicks/drags

		return;
	}

	x = event->x ();
	y = event->y ();

	switch ( caller )
	{
	case 0:	// Mouse press
		oldMouseX = x;
		oldMouseY = y;

		if ( event->modifiers () & Qt::ControlModifier )
		{
			mainwindow->clickMap ( x, y );

			return;
		}

		break;

	case 2:	// Mouse movement
		mainwindow->polymapScroll ( oldMouseX - x, oldMouseY - y );
		break;
	}
}

void MapArea :: mousePressEvent ( QMouseEvent * event )
{
	mouseEventRouter ( event, 0 );
}

void MapArea :: mouseMoveEvent ( QMouseEvent * event )
{
	mouseEventRouter ( event, 2 );
}

void MainWindow :: wheelZoom (
	int		x,
	int		y,
	int		delta
	)
{
	double		aw,
			ah,
			cx,
			cy,
			zd = 0,
			newzoom;


	aw = polymapScrollArea->viewport ()->width ();
	ah = polymapScrollArea->viewport ()->height ();

	cx = ( (double)(x - aw) ) / (mapArea->width () - 2 * aw );
	cy = ( (double)(y - ah) ) / (mapArea->height () - 2 * ah );

	mtMAX ( cx, cx, 0 )
	mtMAX ( cy, cy, 0 )
	mtMIN ( cx, cx, 1 )
	mtMIN ( cy, cy, 1 )

	zd =  zoomMap / 10;

	if ( delta < 0 )
	{
		zd = -zd;
	}

	newzoom = zoomMap + zd;

	if ( newzoom < POLYMAP_ZOOM_MIN )
	{
		newzoom = POLYMAP_ZOOM_MIN;
	}

	if ( newzoom > POLYMAP_ZOOM_MAX )
	{
		newzoom = POLYMAP_ZOOM_MAX;
	}

	if ( zoomMap == newzoom )
	{
		return;
	}


	int		w,
			h,
			nv_h = 0,
			nv_v = 0,	// New positions of scrollbar
			ox,		// Offset coord from visible area
			oy;


	w = rint ( POLYMAP_WIDTH * newzoom );
	h = rint ( POLYMAP_HEIGHT * newzoom );

	ox = x - polymapScrollArea->horizontalScrollBar ()->value ();
	oy = y - polymapScrollArea->verticalScrollBar ()->value ();

	zoomMap = newzoom;

	sliderMapZoom->blockSignals ( true );
	sliderMapZoom->setValue ( rint ( newzoom * 100 ) );
	sliderMapZoom->blockSignals ( false );

	mapArea->setGeometry ( 0, 0, (int)(w + aw * 2), (int)(h + ah * 2) );

	nv_h = rint ( w * cx - ox ) + aw;
	nv_v = rint ( h * cy - oy ) + ah;

	polymapScrollArea->horizontalScrollBar ()->setValue ( nv_h );
	polymapScrollArea->verticalScrollBar ()->setValue ( nv_v );
}

void MapArea :: wheelEvent ( QWheelEvent * event )
{
	if ( event->orientation () == Qt::Horizontal )
	{
		event->ignore ();

		return;
	}

	mainwindow->wheelZoom ( event->x (), event->y (), event->delta () );
}

void MainWindow :: polymapRedraw ()
{
	mapArea->update ();
}

void map_redraw_ini (
	mtPrefItemValue	* piv,
	int		callback_data,
	void		* callback_ptr
	)
{
	mainwindow->polymapRedraw ();
}

void MainWindow :: resetMapSize ()
{
	int		w,
			h;


	w = polymapScrollArea->viewport ()->width ();
	h = polymapScrollArea->viewport ()->height ();

	mapArea->setGeometry ( 0, 0,
		POLYMAP_WIDTH * zoomMap + 2 * w,
		POLYMAP_HEIGHT * zoomMap + 2 * h );

	mapArea->update ();
}

void MainWindow :: resetMapZoomPos ()
{
	int		w,
			h;


	resetMapSize ();

	w = polymapScrollArea->horizontalScrollBar ()->maximum ();
	h = polymapScrollArea->verticalScrollBar ()->maximum ();

	// This ensures the x / y map position is as I want
	polymapScrollArea->horizontalScrollBar ()->setValue ( w / 2 );
	polymapScrollArea->verticalScrollBar ()->setValue ( h / 2 );
}

void MainWindow :: polymapScroll ( int dx, int dy )
{
	int		nx,
			ny;


	nx = dx + polymapScrollArea->horizontalScrollBar ()->value ();
	ny = dy + polymapScrollArea->verticalScrollBar ()->value ();

	polymapScrollArea->horizontalScrollBar ()->setValue ( nx );
	polymapScrollArea->verticalScrollBar ()->setValue ( ny );
}

void MapAreaHolder :: resizeEvent ( QResizeEvent * event )
{
	mainwindow->resetMapSize ();
}

