/*
	Copyright (C) 2011-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "be.h"



static mtPrefs		* prefs_mem;
static char		prefs_filename [ 2048 ];



void prefs_init (
	void		* ptr
	)
{
	mtPrefItem	prefs_table[] = {
	{ PREFS_LAST_DIR,	MTKIT_PREF_TYPE_DIR },

	{ PREFS_WINDOW_X,	MTKIT_PREF_TYPE_INT, "50" },
	{ PREFS_WINDOW_Y,	MTKIT_PREF_TYPE_INT, "50" },
	{ PREFS_WINDOW_W,	MTKIT_PREF_TYPE_INT, "400" },
	{ PREFS_WINDOW_H,	MTKIT_PREF_TYPE_INT, "400" },

	{ PREFS_MAIN_JIGSAW_TABLE,	MTKIT_PREF_TYPE_INT, "1" },
	{ PREFS_MAIN_NODES_HORI,	MTKIT_PREF_TYPE_INT, "10", "Horizontal Mesh Nodes", change_nodes, 0, "6	100", ptr },
	{ PREFS_MAIN_NODES_VERT,	MTKIT_PREF_TYPE_INT, "10", "Vertical Mesh Nodes", change_nodes, 1, "6	100", ptr },

	{ NULL }
	};


	prefs_mem = mtkit_prefs_new ( prefs_table );

	snprintf ( prefs_filename, sizeof ( prefs_filename ), "%s/.%s",
		mtkit_file_home (), BIN_NAME );
	prefs_filename [ sizeof ( prefs_filename ) - 1 ] = 0;
}

void prefs_load ( void )
{
	mtkit_prefs_load ( prefs_mem, prefs_filename );
}

void prefs_save ( void )
{
	mtkit_prefs_save ( prefs_mem, prefs_filename );
}

void prefs_close ( void )
{
	mtkit_prefs_destroy ( prefs_mem );
	prefs_mem = NULL;
}

mtPrefs * prefs_file ( void )
{
	return prefs_mem;
}

void prefs_set_int (
	const char	* key,
	int		value
	)
{
	mtkit_prefs_set_int ( prefs_mem, (char *)key, value );
}

void prefs_set_string (
	const char	* key,
	const char	* value
	)
{
	mtkit_prefs_set_str ( prefs_mem, (char *)key, (char *)value );
}

void prefs_set_double (
	const char	* key,
	double		value
	)
{
	mtkit_prefs_set_double ( prefs_mem, (char *)key, value );
}

int prefs_get_int (
	const char	* key
	)
{
	int		res = 0;


	mtkit_prefs_get_int ( prefs_mem, (char *)key, &res );

	return res;
}

char * prefs_get_string (
	const char	* key
	)
{
	static char	* nothing = "";
	char		* res = nothing;


	mtkit_prefs_get_str ( prefs_mem, (char *)key, &res );

	if ( ! res )
	{
		return nothing;
	}

	return res;
}

double prefs_get_double (
	const char	* key
	)
{
	double		res = 0;


	mtkit_prefs_get_double ( prefs_mem, (char *)key, &res );

	return res;
}
