/*
	Copyright (C) 2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "tk_qt4.h"
#include "icon.xpm"



MainWindow :: MainWindow ()
{
	QWidget		* widget;
	QVBoxLayout	* layout;


	// Preparation

	mtpixel_init ();
	ced_init ();

	prefs_init ( this );

	mtQEX :: fpickInitPrefs ( prefs_file (), (char *)PREFS_MAIN_FPICK_PRE );
	mtQEX :: prefsInitPrefs ( prefs_file () );

	prefs_load ();


	// Widgets

	setWindowTitle ( VERSION );
	this->setWindowIcon ( QPixmap ( icon_xpm ) );

	widget = new QWidget;
	setCentralWidget ( widget );

	canvasWidget = new jigsawCanvas ();
	canvasWidget->setAttribute ( Qt::WA_OpaquePaintEvent );
	canvasWidget->setAttribute ( Qt::WA_NoSystemBackground );

	layout = new QVBoxLayout;
	layout->setMargin ( 0 );
	layout->addWidget ( canvasWidget );

	widget->setLayout ( layout );

	createActions ();
	createMenus ();

	setMinimumSize ( 160, 160 );
	setGeometry ( prefs_get_int ( PREFS_WINDOW_X ),
		prefs_get_int ( PREFS_WINDOW_Y ),
		prefs_get_int ( PREFS_WINDOW_W ),
		prefs_get_int ( PREFS_WINDOW_H ) );

	show ();

	canvasWidget->setJigsawTable ( prefs_get_int (
		PREFS_MAIN_JIGSAW_TABLE ) );
}

MainWindow :: ~MainWindow ()
{
	delete canvasWidget;
	canvasWidget = NULL;

	prefs_set_int ( PREFS_WINDOW_X, geometry().x () );
	prefs_set_int ( PREFS_WINDOW_Y, geometry().y () );
	prefs_set_int ( PREFS_WINDOW_W, geometry().width () );
	prefs_set_int ( PREFS_WINDOW_H, geometry().height () );

	prefs_save ();
	prefs_close ();

	mtpixel_quit ();
}

int MainWindow :: isBusy ()
{
	return 0;			// Not busy
}

void MainWindow :: closeEvent (
	QCloseEvent	* event
	)
{
	if ( isBusy () )
	{
		// Program is busy so we can't stop yet
		event->ignore ();
	}
	else
	{
		// Program is idle so accept closure
		event->accept ();
	}
}

void MainWindow :: createActions ()
{
	actHelpAbout = new QAction ( QIcon :: fromTheme ( "help-about" ),
		tr ( "About" ), this );
	actHelpAbout->setShortcut ( tr ( "F1" ) );
	actHelpAbout->setIconVisibleInMenu ( true );
	connect ( actHelpAbout, SIGNAL ( triggered () ), this,
		SLOT ( pressHelpAbout () ) );

	actJigsaw1 = new QAction ( tr ( "Jigsaw 1" ), this );
	actJigsaw1->setCheckable ( true );
	connect ( actJigsaw1, SIGNAL ( triggered () ), this,
		SLOT ( pressJigsaw1 () ) );

	actJigsaw2 = new QAction ( tr ( "Jigsaw 2" ), this );
	actJigsaw2->setCheckable ( true );
	connect ( actJigsaw2, SIGNAL ( triggered () ), this,
		SLOT ( pressJigsaw2 () ) );

	actJigsaw3 = new QAction ( tr ( "Jigsaw 3" ), this );
	actJigsaw3->setCheckable ( true );
	connect ( actJigsaw3, SIGNAL ( triggered () ), this,
		SLOT ( pressJigsaw3 () ) );

	actJigsaw4 = new QAction ( tr ( "Jigsaw 4" ), this );
	actJigsaw4->setCheckable ( true );
	connect ( actJigsaw4, SIGNAL ( triggered () ), this,
		SLOT ( pressJigsaw4 () ) );

	actJigsaw5 = new QAction ( tr ( "Jigsaw 5" ), this );
	actJigsaw5->setCheckable ( true );
	connect ( actJigsaw5, SIGNAL ( triggered () ), this,
		SLOT ( pressJigsaw5 () ) );

	actJigsaw6 = new QAction ( tr ( "Jigsaw 6" ), this );
	actJigsaw6->setCheckable ( true );
	connect ( actJigsaw6, SIGNAL ( triggered () ), this,
		SLOT ( pressJigsaw6 () ) );

	actJigsaw7 = new QAction ( tr ( "Jigsaw 7" ), this );
	actJigsaw7->setCheckable ( true );
	connect ( actJigsaw7, SIGNAL ( triggered () ), this,
		SLOT ( pressJigsaw7 () ) );

	actJigsaw8 = new QAction ( tr ( "Jigsaw 8" ), this );
	actJigsaw8->setCheckable ( true );
	connect ( actJigsaw8, SIGNAL ( triggered () ), this,
		SLOT ( pressJigsaw8 () ) );

	actJigsaw9 = new QAction ( tr ( "Jigsaw 9" ), this );
	actJigsaw9->setCheckable ( true );
	connect ( actJigsaw9, SIGNAL ( triggered () ), this,
		SLOT ( pressJigsaw9 () ) );

	actJigsaw10 = new QAction ( tr ( "Jigsaw 10" ), this );
	actJigsaw10->setCheckable ( true );
	connect ( actJigsaw10, SIGNAL ( triggered () ), this,
		SLOT ( pressJigsaw10 () ) );

	menuGroupJigsaw = new QActionGroup ( this );
	menuGroupJigsaw->addAction ( actJigsaw1 );
	menuGroupJigsaw->addAction ( actJigsaw2 );
	menuGroupJigsaw->addAction ( actJigsaw3 );
	menuGroupJigsaw->addAction ( actJigsaw4 );
	menuGroupJigsaw->addAction ( actJigsaw5 );
	menuGroupJigsaw->addAction ( actJigsaw6 );
	menuGroupJigsaw->addAction ( actJigsaw7 );
	menuGroupJigsaw->addAction ( actJigsaw8 );
	menuGroupJigsaw->addAction ( actJigsaw9 );
	menuGroupJigsaw->addAction ( actJigsaw10 );

	switch ( prefs_get_int ( PREFS_MAIN_JIGSAW_TABLE ) )
	{
	case 1:		actJigsaw1->setChecked ( true ); break;
	case 2:		actJigsaw2->setChecked ( true ); break;
	case 3:		actJigsaw3->setChecked ( true ); break;
	case 4:		actJigsaw4->setChecked ( true ); break;
	case 5:		actJigsaw5->setChecked ( true ); break;
	case 6:		actJigsaw6->setChecked ( true ); break;
	case 7:		actJigsaw7->setChecked ( true ); break;
	case 8:		actJigsaw8->setChecked ( true ); break;
	case 9:		actJigsaw9->setChecked ( true ); break;
	case 10:	actJigsaw10->setChecked ( true ); break;
	}

	actOptionsNew = new QAction ( tr ( "Start New Puzzle" ), this );
	connect ( actOptionsNew, SIGNAL ( triggered () ), this,
		SLOT ( pressOptionsNew () ) );

	actOptionsJoin = new QAction ( tr ( "Join Pieces" ), this );
	connect ( actOptionsJoin, SIGNAL ( triggered () ), this,
		SLOT ( pressOptionsJoin () ) );

	actOptionsScatter = new QAction ( tr ( "Scatter Pieces" ), this );
	connect ( actOptionsScatter, SIGNAL ( triggered () ), this,
		SLOT ( pressOptionsScatter () ) );

	actOptionsOpenImage = new QAction (
		QIcon :: fromTheme ( "document-open" ),
		tr ( "Open Image" ), this );
	actOptionsOpenImage->setShortcut ( tr ( "Ctrl+O" ) );
	actOptionsOpenImage->setIconVisibleInMenu ( true );
	connect ( actOptionsOpenImage, SIGNAL ( triggered () ), this,
		SLOT ( pressOptionsOpenImage () ) );

	actOptionsFullscreen = new QAction (
		QIcon :: fromTheme ( "view-fullscreen" ),
		tr ( "Full Screen" ), this );
	actOptionsFullscreen->setShortcut ( tr ( "F11" ) );
	actOptionsFullscreen->setIconVisibleInMenu ( true );
	connect ( actOptionsFullscreen, SIGNAL ( triggered () ), this,
		SLOT ( pressOptionsFullscreen () ) );

	actOptionsPrefs = new QAction (
		QIcon :: fromTheme ( "preferences-other" ),
		tr ( "Preferences" ), this );
	actOptionsPrefs->setShortcut ( tr ( "Ctrl+P" ) );
	actOptionsPrefs->setIconVisibleInMenu  ( true );
	connect ( actOptionsPrefs, SIGNAL ( triggered () ), this,
		SLOT ( pressOptionsPrefs () ) );

	actOptionsQuit = new QAction (
		QIcon :: fromTheme ( "application-exit" ),
		tr ( "Quit" ), this );
	actOptionsQuit->setShortcut ( tr ( "Ctrl+Q" ) );
	actOptionsQuit->setIconVisibleInMenu ( true );
	connect ( actOptionsQuit, SIGNAL ( triggered () ), this,
		SLOT ( pressOptionsQuit () ) );
}

void MainWindow :: createMenus ()
{
	menuOptions = menuBar ()->addMenu ( tr ( "&Options" ) );
	menuOptions->setTearOffEnabled ( true );
	menuOptions->addAction ( actOptionsNew );
	menuOptions->addAction ( actOptionsJoin );
	menuOptions->addAction ( actOptionsScatter );
	menuOptions->addSeparator ();
	menuOptions->addAction ( actOptionsOpenImage );
	menuOptions->addSeparator ();
	menuOptions->addAction ( actOptionsFullscreen );
	menuOptions->addAction ( actOptionsPrefs );
	menuOptions->addSeparator ();
	menuOptions->addAction ( actOptionsQuit );

	menuJigsaw = menuBar ()->addMenu ( tr ( "&Jigsaw" ) );
	menuJigsaw->setTearOffEnabled ( true );
	menuJigsaw->addAction ( actJigsaw1 );
	menuJigsaw->addAction ( actJigsaw2 );
	menuJigsaw->addAction ( actJigsaw3 );
	menuJigsaw->addAction ( actJigsaw4 );
	menuJigsaw->addAction ( actJigsaw5 );
	menuJigsaw->addAction ( actJigsaw6 );
	menuJigsaw->addAction ( actJigsaw7 );
	menuJigsaw->addAction ( actJigsaw8 );
	menuJigsaw->addAction ( actJigsaw9 );
	menuJigsaw->addAction ( actJigsaw10 );

	menuHelp = menuBar ()->addMenu ( tr ( "&Help" ) );
	menuHelp->addAction ( actHelpAbout );
}

void MainWindow :: pressHelpAbout ()
{
	QMessageBox :: about ( this, tr ( "About" ),
		tr ( VERSION"\n"
	"\n"
	"Copyright (C) 2009-2013 Mark Tyler.\n"
	"\n"
	"This program is free software: you can redistribute it and/or modify "
	"it under the terms of the GNU General Public License as published by "
	"the Free Software Foundation, either version 3 of the License, or "
	"(at your option) any later version.\n"
	"\n"
	"This program is distributed in the hope that it will be useful, "
	"but WITHOUT ANY WARRANTY; without even the implied warranty of "
	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
	"GNU General Public License for more details.\n"
	"\n"
	"You should have received a copy of the GNU General Public License "
	"along with this program.  If not, see http://www.gnu.org/licenses/\n"
		) );
}

void MainWindow :: pressJigsaw1 () { canvasWidget->setJigsawTable ( 1 ); }
void MainWindow :: pressJigsaw2 () { canvasWidget->setJigsawTable ( 2 ); }
void MainWindow :: pressJigsaw3 () { canvasWidget->setJigsawTable ( 3 ); }
void MainWindow :: pressJigsaw4 () { canvasWidget->setJigsawTable ( 4 ); }
void MainWindow :: pressJigsaw5 () { canvasWidget->setJigsawTable ( 5 ); }
void MainWindow :: pressJigsaw6 () { canvasWidget->setJigsawTable ( 6 ); }
void MainWindow :: pressJigsaw7 () { canvasWidget->setJigsawTable ( 7 ); }
void MainWindow :: pressJigsaw8 () { canvasWidget->setJigsawTable ( 8 ); }
void MainWindow :: pressJigsaw9 () { canvasWidget->setJigsawTable ( 9 ); }
void MainWindow :: pressJigsaw10 () { canvasWidget->setJigsawTable ( 10 ); }

void MainWindow :: pressOptionsNew ()
{
	int		ret;


	ret = QMessageBox :: question ( this, tr ( "Question" ),
		tr ( "Do you really want to start a new puzzle?" ),
		QMessageBox::Yes | QMessageBox::No,
		QMessageBox::No );

	if ( ret == QMessageBox::Yes )
	{
		setMesh ( prefs_get_int ( PREFS_MAIN_NODES_HORI ),
			prefs_get_int ( PREFS_MAIN_NODES_VERT ) );
	}
}

void MainWindow :: pressOptionsJoin ()
{
	int		ret;


	ret = QMessageBox :: question ( this, tr ( "Question" ),
		tr ( "Do you really want to join the pieces?" ),
		QMessageBox::Yes | QMessageBox::No,
		QMessageBox::No );

	if ( ret == QMessageBox::Yes )
	{
		canvasWidget->joinJigsaw ();
	}
}

void MainWindow :: pressOptionsScatter ()
{
	int		ret;


	ret = QMessageBox :: question ( this, tr ( "Question" ),
		tr ( "Do you really want to scatter the pieces?" ),
		QMessageBox::Yes | QMessageBox::No,
		QMessageBox::No );

	if ( ret == QMessageBox::Yes )
	{
		canvasWidget->scatterJigsaw ();
	}
}

void MainWindow :: pressOptionsOpenImage ()
{
	QString filename = QFileDialog::getOpenFileName ( this,
		tr ( "Open Image" ),
		prefs_get_string ( PREFS_LAST_DIR ),
		tr ( "Image Files (*.png *.gif *.bmp *.jpg);;All Files (*)" ),
		NULL,
		QFileDialog::DontUseNativeDialog );

	canvasWidget->setJigsawImage ( filename );
}

void MainWindow :: pressOptionsFullscreen ()
{
	if ( isFullScreen () )
	{
		showNormal ();
	}
	else
	{
		showFullScreen ();
	}
}

void MainWindow :: pressOptionsPrefs ()
{
	qexPrefsWindow ( prefs_file (), "Preferences" );
}

void MainWindow :: pressOptionsQuit ()
{
	if ( isBusy () )
	{
		return;
	}

	close ();
}

void MainWindow :: setMesh (
	int	hori,
	int	vert
	)
{
	canvasWidget->setJigsawMesh ( hori, vert );
}

void change_nodes (
	mtPrefItemValue	* piv,
	int		callback_data,
	void		* callback_ptr
	)
{
	MainWindow	* main_window = (MainWindow *)callback_ptr;


	main_window->setMesh ( prefs_get_int ( PREFS_MAIN_NODES_HORI ),
		prefs_get_int ( PREFS_MAIN_NODES_VERT ) );
}

int main (
	int		argc,
	char		** argv
	)
{
	be_cline ( argc, argv );


	QApplication	app ( argc, argv );
	MainWindow	window;


	return app.exec ();
}
