/*
	Copyright (C) 2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <mtpixel.h>



static int	exit_val	= 0,
		ofile_type	= MTPIXEL_FILE_TYPE_PNG,
		ofile_num	= 5
		;



static int print_version (
	mtArg		* mtarg,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	printf ( "mtimcp - Part of %s\n", VERSION );
	printf ( "Mark Tyler's IMage CoPy\n\n" );

	return 1;			// Stop parsing
}

static int print_help (
	mtArg		* mtarg,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	print_version ( mtarg, arg, argc, argv );

	printf (
	"Usage: mtimcp [ [OPTION] [FILE] ]...\n"
	"\n"
	"Options:\n"
	"  --help               Output this help.\n"
	"  --version            Output version information.\n"
	"  -bmp                 Output file = BMP.\n"
	"  -gif                 Output file = GIF.\n"
	"  -jpeg                Output file = JPEG.\n"
	"  -png                 Output file = PNG (default).\n"
	"\n"
	"The new output file type is appended to the filename.\n"
	"No file is ever overwritten or deleted.\n"
	"\n"
		);

	return 1;			// Stop parsing
}

static int file_func (
	char		* filename
	)
{
	mtImage		* img;
	char		* new_filename = NULL,
			* strp
			;


	img = mtpixel_image_load ( filename );

	if ( ! img )
	{
		fprintf ( stderr, "Unable to load %s\n", filename );

		goto error;
	}

	new_filename = calloc ( strlen ( filename ) + 8, 1 );

	if ( ! new_filename )
	{
		fprintf ( stderr, "Unable to create new filename\n" );

		goto error;
	}

	strcpy ( new_filename, filename );

	switch ( ofile_type )
	{
	case MTPIXEL_FILE_TYPE_PNG:	strp = ".png";	break;
	case MTPIXEL_FILE_TYPE_BMP:	strp = ".bmp";	break;
	case MTPIXEL_FILE_TYPE_JPEG:	strp = ".jpeg";	break;
	case MTPIXEL_FILE_TYPE_GIF:	strp = ".gif";	break;

	default:
		fprintf ( stderr, "Invalid file type: %i\n", ofile_type );

		goto error;
	}

	strcat ( new_filename, strp );

	if ( 0 == mtkit_file_valid ( new_filename ) )
	{
		fprintf ( stderr, "File already exists: %s\n", new_filename );

		goto error;
	}

	if ( mtpixel_image_save ( img, new_filename, ofile_type, ofile_num ) )
	{
		fprintf ( stderr, "Unable to save %s, type = %i, num = %i\n",
			new_filename, ofile_type, ofile_num );

		goto error;
	}

	free ( new_filename );
	mtpixel_image_destroy ( img );

	return 0;

error:
	free ( new_filename );
	mtpixel_image_destroy ( img );
	exit_val = 1;

	return 1;
}

static int error_func (
	int		error,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	fprintf ( stderr, "error_func: Argument ERROR! - num = %i arg = %i/%i",
		error, arg, argc );

	if ( arg < argc )
	{
		fprintf ( stderr, " '%s'", argv[arg] );
	}

	fprintf ( stderr, "\n" );

	return 0;			// Keep parsing
}

int main (
	int		argc,
	char		* argv[]
	)
{
	int		tmp;

	mtArg		arg_list[] = {
	{ "-help",	MTKIT_ARG_SWITCH, &tmp, 0, print_help },
	{ "-version",	MTKIT_ARG_SWITCH, &tmp, 0, print_version },
	{ "bmp",	MTKIT_ARG_SWITCH, &ofile_type, MTPIXEL_FILE_TYPE_BMP },
	{ "fileval",	MTKIT_ARG_INT,	&ofile_num },
	{ "gif",	MTKIT_ARG_SWITCH, &ofile_type, MTPIXEL_FILE_TYPE_GIF },
	{ "jpeg",	MTKIT_ARG_SWITCH, &ofile_type, MTPIXEL_FILE_TYPE_JPEG },
	{ "png",	MTKIT_ARG_SWITCH, &ofile_type, MTPIXEL_FILE_TYPE_PNG },
	{ NULL }
	};


	if ( mtpixel_init () )
	{
		fprintf ( stderr, "Unable to start mtPixel.\n\n" );

		exit_val = 1;
	}
	else
	{
		mtkit_arg_parse ( argc, argv, arg_list, file_func, error_func );

		mtpixel_quit ();
	}

	return exit_val;
}
