/*
	Copyright (C) 2004-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#include <mtcelledit.h>



enum	// Column numbers for each field in results sheet
{
	RAFT_COL_NAME		= 1,
	RAFT_COL_FILES		= 2,
	RAFT_COL_FILES_PERCENT	= 3,
	RAFT_COL_BYTES		= 4,
	RAFT_COL_MB		= 5,
	RAFT_COL_MB_PERCENT	= 6,
	RAFT_COL_SUBDIRS	= 7,
	RAFT_COL_OTHER		= 8,

	RAFT_COL_TOTAL		= 9
};



#define MAX_COMBO_ITEMS		16

#define PATH_LEN		2048
#define PATH_LEN_SHORT		20
#define PATH_LEN_SHORT_BUF	128

#define PREFS_WINDOW_X		"window_x"
#define PREFS_WINDOW_Y		"window_y"
#define PREFS_WINDOW_W		"window_w"
#define PREFS_WINDOW_H		"window_h"


typedef int (* raftScanFunc) (
	CedSheet	* sheet,
	int		row,		// Row calculated in sheet
	void		* user_data
	);
	// 0 = Keep scanning
	// 1 = Stop



// 0 = success, NULL = fail unless otherwise stated

void prefs_init ( void );	// Setup memory, filename, etc.
void prefs_load ( void );	// Load prefs file from disk into memory
void prefs_save ( void );	// Save current prefs from memory to disk
void prefs_close ( void );	// Free all memory created in prefs_init ()

void prefs_set_int (
	const char	* key,
	int		value
	);

void prefs_set_string (
	const char	* key,
	const char	* value
	);

void prefs_set_double (
	const char	* key,
	double		value
	);

int prefs_get_int (
	const char	* key
	);

char * prefs_get_string (
	const char	* key
	);

double prefs_get_double (
	const char	* key
	);

int raft_scan_sheet (
	const char	* path,		// Path to scan all files & subdirs
	CedSheet	** sheet,	// Put results here
	raftScanFunc	callback,	// Option callback for each subdir
	void		* user_data	// Passed to raftScanFunc
	);
	// -1 = Error
	//  0 = Success - sheet created
	//  1 = User termination

const char * raft_cline (		// Parse command line
	int		argc,
	char		** argv
	);
	// Returns pointer to path, or NULL if none was passed

int raft_path_check (			// Check path, put into buffer
	const char	* path,
	char		* buf		// PATH_LEN sized
	);
	// 0 = buf contains a valid path.
	// 1 = Error.

int raft_path_merge (			// Merge the current path and selected row
	char		* buf,		// PATH_LEN size for result
	const char	* path,		// Must have a / at the end
	CedSheet	* sheet,
	int		row
	);

char * raft_get_clipboard (		// Create UTF8 text for clipboard
	CedSheet	* sheet		// from this sheet
	);
	// Caller must free any result after use.
